import pygraphviz as pgv


def test_del():
    A = pgv.AGraph()
    A.add_node(1, foo="bar")
    # For some reasons after porting to Python 3 clear often cause infinite loop
    A.delete_node("1")
    assert len(A) == 0


def test_clear_node_with_attributes():
    A = pgv.AGraph()
    A.add_node(1, foo="bar")
    # For some reasons after porting to Python 3 clear often cause infinite loop
    A.clear()
    assert len(A) == 0
    assert A.nodes() == []
    assert A.node_attr.keys() in ([], ["label"])


def test_clear_graph_attributes():
    A = pgv.AGraph()
    A.add_node(1, foo="bar")
    A.graph_attr.update(landscape="true", ranksep="0.1")
    # For some reasons after porting to Python 3 clear often cause infinite loop
    A.clear()
    assert len(A) == 0
    assert A.nodes() == []
    assert A.node_attr.keys() in ([], ["label"])
    assert A.graph_attr.keys() == []



import pygraphviz as pgv

stringify = pgv.testing.stringify


long_html_string = """<<TABLE BORDER=0>
  <TR>
      <TD> meow </TD>
  </TR>
  <TR>
      <TD>
        <TABLE>
          <TR>
          <TD align=left>Count</TD>
          <TD align=right> 4 </TD>
          </TR>
          <TR>
          <TD align=left>Earliest Run</TD>
          <TD align=right> yesterday </TD>
          </TR>
          <TR>
          <TD align=left>Latest Run</TD>
          <TD align=right> tomorrow </TD>
          </TR>
          <TR>
          <TD align=left>Avg Runtime</TD>
          <TD align=right> 4 seconds </TD>
          </TR>
          <TR>
          <TD align=left>Avg Failure Rate</TD>
          <TD align=right> 38.1% </TD>
          </TR>
        </TABLE>
      </TD>
  </TR>
</TABLE>>"""


def test_long_html_string():
    G = pgv.AGraph(label="<Hello<BR/>Graph>")
    G.add_node("a", label=long_html_string)
    s = G.add_subgraph("b", label="<Hello<BR/>Subgraph>")
    s.add_node("sa", label="<Hello<BR/>Subgraph Node b>")
    G.add_edge("a", "b", label="<Hello<BR/>Edge>")
    ans = """strict graph {{
              graph [label=<Hello<BR/>Graph>];
              {{
                graph [label=<Hello<BR/>Subgraph>];
                sa     [label=<Hello<BR/>Subgraph Node b>];
              }}
              a  [label={0}];
              a -- b   [label=<Hello<BR/>Edge>];
            }}""".format(
        long_html_string
    )
    assert stringify(G) == " ".join(ans.split())


def test_html():
    G = pgv.AGraph(label="<Hello<BR/>Graph>")
    G.add_node("a", label="<Hello<BR/>Node>")
    s = G.add_subgraph("b", label="<Hello<BR/>Subgraph>")
    s.add_node("sa", label="<Hello<BR/>Subgraph Node b>")
    G.add_edge("a", "b", label="<Hello<BR/>Edge>")
    ans = """strict graph {
      graph [label=<Hello<BR/>Graph>];
      {
        graph [label=<Hello<BR/>Subgraph>];
        sa [label=<Hello<BR/>Subgraph Node b>];
      }
      a  [label=<Hello<BR/>Node>];
      a -- b [label=<Hello<BR/>Edge>];
    }"""
    assert stringify(G) == " ".join(ans.split())



import pygraphviz as pgv

stringify = pgv.testing.stringify


def test_edge_attributes():
    A = pgv.AGraph()
    A.add_edge(1, 2, label="test", spam="eggs")
    ans = """strict graph { 1 -- 2 [label=test, spam=eggs]; }"""
    assert stringify(A) == " ".join(ans.split())


def test_edge_attributes2():
    A = pgv.AGraph()
    A.add_edge(1, 2)
    one = A.get_edge(1, 2)
    one.attr["label"] = "test"
    one.attr["spam"] = "eggs"
    assert "label" in one.attr
    assert one.attr["label"] == "test"
    assert sorted(one.attr.keys()) == ["label", "spam"]

    ans = """strict graph { 1 -- 2 [label=test, spam=eggs]; }"""
    assert stringify(A) == " ".join(ans.split())

    one.attr["label"] = ""
    one.attr["spam"] = ""
    ans = """strict graph { 1 -- 2; }"""
    assert stringify(A) == " ".join(ans.split())

    one.attr["label"] = "test"
    del one.attr["label"]
    ans = """strict graph { 1 -- 2; }"""
    assert stringify(A) == " ".join(ans.split())


def test_anonymous_edges():
    text_graph = (
        b"""graph test {\n a -- b [label="edge1"];\n a -- b [label="edge2"];\n }"""
    )

    import os
    import tempfile

    fd, fname = tempfile.mkstemp()
    os.write(fd, text_graph)
    os.close(fd)
    A = pgv.AGraph(fname)

    ans = """graph test { a -- b [label=edge1]; a -- b [label=edge2]; }"""
    assert stringify(A) == " ".join(ans.split())
    os.unlink(fname)


def test_edge_attribute_update():
    A = pgv.AGraph(strict=True)
    A.add_edge(1, 2, label="test", spam="eggs")
    A.add_edge(1, 2, label="update", spam="")
    ans = """strict graph { 1 -- 2 [label=update]; }"""
    assert stringify(A) == " ".join(ans.split())


def test_edge_attribute_update_nonstrict():
    A = pgv.AGraph(strict=False)
    A.add_edge(1, 2, label="test", spam="eggs", key="one")
    A.add_edge(1, 2, label="update", spam="", key="one")
    ans = """graph { 1 -- 2 [key=one, label=update]; }"""
    assert stringify(A) == " ".join(ans.split())



import pygraphviz as pgv

stringify = pgv.testing.stringify


def test_subgraph():
    G = pgv.AGraph()
    G.add_edge(1, 2)
    s = G.add_subgraph((1, 2), label="foo")
    ans = """strict graph { { graph [label=foo]; 1 -- 2; } }"""
    assert stringify(G) == ans


def test_subgraph_cluster():
    G = pgv.AGraph(label="foo")
    s = G.subgraph("cluster_a", label="<Hello<BR/>World>")
    s.add_node("sa")
    G.add_node("a")
    ans = """strict graph { graph [label=foo];
             { graph [label=<Hello<BR/>World>]; sa; }
             a; }"""
    assert stringify(G) == " ".join(ans.split())


def test_subgraph_cluster_attribute():
    G = pgv.AGraph()
    s = G.subgraph(name="cluster_a")
    s.node_attr["foo"] = "bar"
    G.add_node("a")
    G.node_attr["foo"] = "baz"
    ans = """strict graph { node [foo=baz];
             subgraph cluster_a {
               graph [foo=bar];
             }
             a; }"""
    assert stringify(G) == " ".join(ans.split())



"""
Example showing use of unicode and UTF-8 encoding.
"""


import pygraphviz as pgv

# specify UTF-8 encoding (it is the default)
A = pgv.AGraph(encoding="UTF-8")

# nodes, attributes, etc can be strings or unicode
A.add_node(1, label="plain string")
A.add_node(2, label="unicode")

# you can enter unicode text as
hello = "Здравствуйте!"
A.add_node(3, label=hello)

# or using unicode code points
hello = "\u0417\u0434\u0440\u0430\u0432\u0441\u0442\u0432\u0443\u0439\u0442\u0435!"
A.add_node(hello)  # unicode node label

goodbye = "До свидания"
A.add_edge(1, hello, key=goodbye)

A.add_edge("שלום", hello)
# A.add_edge(1,3,hello="こんにちは / ｺﾝﾆﾁﾊ")
A.add_edge(1, "こんにちは")

print(A)  # print to screen
A.write("utf8.dot")  # write to simple.dot
